@@ -1,5 +1,5 @@
 /* poppler-link.cc: qt interface to poppler
- * Copyright (C) 2006-2007, 2013, 2016-2021, Albert Astals Cid
+ * Copyright (C) 2006-2007, 2013, 2016-2021, 2024, Albert Astals Cid
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
  * Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
  * Copyright (C) 2012, Tobias Koenig <tokoe@kdab.com>
@@ -7,6 +7,7 @@
  * Copyright (C) 2018 Intevation GmbH <intevation@intevation.de>
  * Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
  * Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
  * Adapting code from
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>
  *
@@ -78,6 +79,10 @@ LinkOCGStatePrivate::~LinkOCGStatePrivate() = default;
 
 LinkHidePrivate::~LinkHidePrivate() = default;
 
+LinkResetFormPrivate::~LinkResetFormPrivate() = default;
+
+LinkSubmitFormPrivate::~LinkSubmitFormPrivate() = default;
+
 class LinkGotoPrivate : public LinkPrivate
 {
 public:
@@ -151,17 +156,17 @@ LinkSoundPrivate::~LinkSoundPrivate()
 class LinkRenditionPrivate : public LinkPrivate
 {
 public:
-    explicit LinkRenditionPrivate(const QRectF &area, ::MediaRendition *rendition, ::LinkRendition::RenditionOperation operation, const QString &script, const Ref ref);
+    explicit LinkRenditionPrivate(const QRectF &area, std::unique_ptr<::MediaRendition> &&rendition, ::LinkRendition::RenditionOperation operation, const QString &script, const Ref ref);
     ~LinkRenditionPrivate() override;
 
-    MediaRendition *rendition;
+    std::unique_ptr<MediaRendition> rendition;
     LinkRendition::RenditionAction action;
     QString script;
     Ref annotationReference;
 };
 
-LinkRenditionPrivate::LinkRenditionPrivate(const QRectF &area, ::MediaRendition *r, ::LinkRendition::RenditionOperation operation, const QString &javaScript, const Ref ref)
-    : LinkPrivate(area), rendition(r ? new MediaRendition(r) : nullptr), action(LinkRendition::PlayRendition), script(javaScript), annotationReference(ref)
+LinkRenditionPrivate::LinkRenditionPrivate(const QRectF &area, std::unique_ptr<::MediaRendition> &&r, ::LinkRendition::RenditionOperation operation, const QString &javaScript, const Ref ref)
+    : LinkPrivate(area), rendition(r ? new MediaRendition(std::move(r)) : nullptr), action(LinkRendition::PlayRendition), script(javaScript), annotationReference(ref)
 {
     switch (operation) {
     case ::LinkRendition::NoRendition:
@@ -182,10 +187,7 @@ LinkRenditionPrivate::LinkRenditionPrivate(const QRectF &area, ::MediaRendition
     }
 }
 
-LinkRenditionPrivate::~LinkRenditionPrivate()
-{
-    delete rendition;
-}
+LinkRenditionPrivate::~LinkRenditionPrivate() = default;
 
 class LinkJavaScriptPrivate : public LinkPrivate
 {
@@ -237,29 +239,31 @@ LinkDestination::LinkDestination(const LinkDestinationData &data) : d(new LinkDe
         d->name = QString::fromLatin1(data.namedDest->c_str());
     }
 
-    if (!ld)
+    if (!ld) {
         return;
+    }
 
-    if (ld->getKind() == ::destXYZ)
+    if (ld->getKind() == ::destXYZ) {
         d->kind = destXYZ;
-    else if (ld->getKind() == ::destFit)
+    } else if (ld->getKind() == ::destFit) {
         d->kind = destFit;
-    else if (ld->getKind() == ::destFitH)
+    } else if (ld->getKind() == ::destFitH) {
         d->kind = destFitH;
-    else if (ld->getKind() == ::destFitV)
+    } else if (ld->getKind() == ::destFitV) {
         d->kind = destFitV;
-    else if (ld->getKind() == ::destFitR)
+    } else if (ld->getKind() == ::destFitR) {
         d->kind = destFitR;
-    else if (ld->getKind() == ::destFitB)
+    } else if (ld->getKind() == ::destFitB) {
         d->kind = destFitB;
-    else if (ld->getKind() == ::destFitBH)
+    } else if (ld->getKind() == ::destFitBH) {
         d->kind = destFitBH;
-    else if (ld->getKind() == ::destFitBV)
+    } else if (ld->getKind() == ::destFitBV) {
         d->kind = destFitBV;
+    }
 
-    if (!ld->isPageRef())
+    if (!ld->isPageRef()) {
         d->pageNum = ld->getPageNum();
-    else {
+    } else {
         const Ref ref = ld->getPageRef();
         d->pageNum = data.doc->doc->findPage(ref);
     }
@@ -284,12 +288,14 @@ LinkDestination::LinkDestination(const LinkDestinationData &data) : d(new LinkDe
             d->top = topAux / (double)page->getCropHeight();
             d->right = rightAux / (double)page->getCropWidth();
             d->bottom = bottomAux / (double)page->getCropHeight();
-        } else
+        } else {
             d->pageNum = 0;
+        }
     }
 
-    if (deleteDest)
+    if (deleteDest) {
         delete ld;
+    }
 }
 
 LinkDestination::LinkDestination(const QString &description) : d(new LinkDestinationPrivate)
@@ -385,8 +391,9 @@ QString LinkDestination::destinationName() const
 
 LinkDestination &LinkDestination::operator=(const LinkDestination &other)
 {
-    if (this == &other)
+    if (this == &other) {
         return *this;
+    }
 
     d = other.d;
     return *this;
@@ -567,7 +574,12 @@ SoundObject *LinkSound::sound() const
 
 // LinkRendition
 LinkRendition::LinkRendition(const QRectF &linkArea, ::MediaRendition *rendition, int operation, const QString &script, const Ref &annotationReference) // clazy:exclude=function-args-by-value
-    : Link(*new LinkRenditionPrivate(linkArea, rendition, static_cast<enum ::LinkRendition::RenditionOperation>(operation), script, annotationReference))
+    : LinkRendition(linkArea, std::unique_ptr<::MediaRendition>(rendition), operation, script, annotationReference)
+{
+}
+
+LinkRendition::LinkRendition(const QRectF &linkArea, std::unique_ptr<::MediaRendition> &&rendition, int operation, const QString &script, const Ref annotationReference)
+    : Link(*new LinkRenditionPrivate(linkArea, std::move(rendition), static_cast<enum ::LinkRendition::RenditionOperation>(operation), script, annotationReference))
 {
 }
 
@@ -581,7 +593,7 @@ Link::LinkType LinkRendition::linkType() const
 MediaRendition *LinkRendition::rendition() const
 {
     Q_D(const LinkRendition);
-    return d->rendition;
+    return d->rendition.get();
 }
 
 LinkRendition::RenditionAction LinkRendition::action() const
@@ -687,4 +699,58 @@ bool LinkHide::isShowAction() const
     Q_D(const LinkHide);
     return d->isShow;
 }
+
+// LinkResetForm
+LinkResetForm::LinkResetForm(LinkResetFormPrivate *lrfp) : Link(*lrfp) { }
+
+LinkResetForm::~LinkResetForm() { }
+
+Link::LinkType LinkResetForm::linkType() const
+{
+    return ResetForm;
+}
+
+// LinkSubmitForm
+// static assertions to ensure flags from Link.h match those from poppler-link.h
+static_assert(static_cast<int>(LinkSubmitForm::NoOpFlag) == static_cast<int>(::LinkSubmitForm::NoOpFlag), "NoOpFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::ExcludeFlag) == static_cast<int>(::LinkSubmitForm::ExcludeFlag), "ExcludeFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::IncludeNoValueFieldsFlag) == static_cast<int>(::LinkSubmitForm::IncludeNoValueFieldsFlag), "IncludeNoValueFieldsFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::ExportFormatFlag) == static_cast<int>(::LinkSubmitForm::ExportFormatFlag), "ExportFormatFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::GetMethodFlag) == static_cast<int>(::LinkSubmitForm::GetMethodFlag), "GetMethodFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::SubmitCoordinatesFlag) == static_cast<int>(::LinkSubmitForm::SubmitCoordinatesFlag), "SubmitCoordinatesFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::XFDFFlag) == static_cast<int>(::LinkSubmitForm::XFDFFlag), "XFDFFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::IncludeAppendSavesFlag) == static_cast<int>(::LinkSubmitForm::IncludeAppendSavesFlag), "IncludeAppendSavesFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::IncludeAnnotationsFlag) == static_cast<int>(::LinkSubmitForm::IncludeAnnotationsFlag), "IncludeAnnotationsFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::SubmitPDFFlag) == static_cast<int>(::LinkSubmitForm::SubmitPDFFlag), "SubmitPDFFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::CanonicalFormatFlag) == static_cast<int>(::LinkSubmitForm::CanonicalFormatFlag), "CanonicalFormatFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::ExclNonUserAnnotsFlag) == static_cast<int>(::LinkSubmitForm::ExclNonUserAnnotsFlag), "ExclNonUserAnnotsFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::ExclFKeyFlag) == static_cast<int>(::LinkSubmitForm::ExclFKeyFlag), "ExclFKeyFlag does not match");
+static_assert(static_cast<int>(LinkSubmitForm::EmbedFormFlag) == static_cast<int>(::LinkSubmitForm::EmbedFormFlag), "EmbedFormFlag does not match");
+
+LinkSubmitForm::LinkSubmitForm(LinkSubmitFormPrivate *lsfp) : Link(*lsfp) { }
+
+LinkSubmitForm::~LinkSubmitForm() { }
+
+Link::LinkType LinkSubmitForm::linkType() const
+{
+    return SubmitForm;
+}
+
+QVector<int> LinkSubmitForm::getFieldIds() const
+{
+    Q_D(const LinkSubmitForm);
+    return d->m_fieldIds;
+}
+
+QString LinkSubmitForm::getUrl() const
+{
+    Q_D(const LinkSubmitForm);
+    return d->m_url;
+}
+
+LinkSubmitForm::SubmitFormFlags LinkSubmitForm::getFlags() const
+{
+    Q_D(const LinkSubmitForm);
+    return d->m_flags;
+}
 }
