@@ -346,6 +346,29 @@ Link *PageData::convertLinkActionToLink(::LinkAction *a, DocumentData *parentDoc
         LinkResetFormPrivate *lrfp = new LinkResetFormPrivate(linkArea, qStringFields, lrf->getExclude());
         popplerLink = new LinkResetForm(lrfp);
     } break;
+    case actionSubmitForm: {
+        ::LinkSubmitForm *lsf = (::LinkSubmitForm *)a;
+        const std::vector<std::string> &stdStringFields = lsf->getFields();
+        QVector<int> fieldIds;
+        fieldIds.reserve(stdStringFields.size());
+        Form *form = parentDoc->doc->getCatalog()->getForm();
+        for (const std::string &fieldStr : stdStringFields) {
+            ::FormField *field = form->findFieldByFullyQualifiedNameOrRef(fieldStr);
+            if (!field->getNoExport()) {
+                int numWidgets = field->getNumWidgets();
+                for (int i = 0; i < numWidgets; i++) {
+                    ::FormWidget *widget = field->getWidget(i);
+                    if (widget) {
+                        fieldIds.append(field->getWidget(i)->getID());
+                    }
+                }
+            }
+        }
+        QString qStringUrl = QString::fromStdString(lsf->getUrl());
+        LinkSubmitForm::SubmitFormFlags qFlags = static_cast<LinkSubmitForm::SubmitFormFlags>(lsf->getFlags());
+        LinkSubmitFormPrivate *lsfp = new LinkSubmitFormPrivate(linkArea, fieldIds, qStringUrl, qFlags);
+        popplerLink = new LinkSubmitForm(lsfp);
+    } break;
 
     case actionUnknown:
         break;
